package config

import (
	"encoding/binary"
	"errors"
	"fmt"
	"net"
	"time"

	"git.faercol.me/faercol/http-boot-config/config/efivar"
	"github.com/google/uuid"
)

const DefaultDiscoverAddr = "[ff02::1%enp6s0]:42"
const DefaultDiscoveryTimeout = 30 * time.Second

type MulticastConfig struct {
	Group   net.IP
	SrcAddr net.IP
	Port    int
}

type Autodiscovery struct {
	DiscoveryAddr string
	Timeout       time.Duration
}

type AppConfig struct {
	Enrolled  bool
	Multicast MulticastConfig
	ClientID  uuid.UUID
	Discovery Autodiscovery
}

func (a *AppConfig) getDataFromEFIVars() error {
	srcAddr, err := efivar.GetVar(efivar.VendorID, efivar.VarBootIP)
	if err != nil {
		return err
	}
	a.Multicast.SrcAddr = net.IP(srcAddr)

	mcastGroup, err := efivar.GetVar(efivar.VendorID, efivar.VarBootGroup)
	if err != nil {
		return err
	}
	a.Multicast.Group = net.IP(mcastGroup)

	mcastPortRaw, err := efivar.GetVar(efivar.VendorID, efivar.VarBootPort)
	if err != nil {
		return err
	}
	mcastPort := binary.LittleEndian.Uint16(mcastPortRaw)
	a.Multicast.Port = int(mcastPort)

	clientID, err := efivar.GetVar(efivar.VendorID, efivar.VarBootUUID)
	if err != nil {
		return err
	}
	a.ClientID = uuid.UUID(clientID)

	return nil
}

func Get() (AppConfig, error) {
	conf := AppConfig{Enrolled: true, Discovery: Autodiscovery{Timeout: DefaultDiscoveryTimeout, DiscoveryAddr: DefaultDiscoverAddr}}

	if err := conf.getDataFromEFIVars(); err != nil {
		if errors.Is(err, efivar.ErrNotFound) {
			conf.Enrolled = false
		} else {
			return conf, fmt.Errorf("failed to read EFI config: %w", err)
		}
	}

	return conf, nil
}
